"use strict";
var Promise = require('bluebird');
var request = require('request-promise');
var url = require('url');
var _ = require('lodash');
var fs = require('fs');
var path = require('path');
var cookie = require('cookie');
var util = require('util');
var xmldoc = require('xmldoc');
var Cache_1 = require('./../../utils/Cache');
var consts = require('./../../Consts');
var AdfsCredentials = (function () {
    function AdfsCredentials(_siteUrl, _authOptions) {
        this._siteUrl = _siteUrl;
        this._authOptions = _.extend({}, _authOptions);
        if (this._authOptions.domain !== undefined) {
            this._authOptions.username = this._authOptions.domain + "\\" + this._authOptions.username;
        }
    }
    AdfsCredentials.prototype.getAuth = function () {
        var _this = this;
        var adfsHost = url.parse(this._authOptions.adfsUrl).host;
        var siteUrlParsed = url.parse(this._siteUrl);
        var usernameMixedUrl = "https://" + adfsHost + "/adfs/services/trust/13/usernamemixed";
        var cacheKey = util.format('%s@%s', siteUrlParsed.host, this._authOptions.username);
        var cachedCookie = AdfsCredentials.CookieCache.get(cacheKey);
        if (cachedCookie) {
            return Promise.resolve({
                headers: {
                    'Cookie': cachedCookie
                }
            });
        }
        var samlTemplate = fs.readFileSync(path.join(__dirname, '..', '..', '..', '..', 'templates', 'adfs_saml_wsfed.tmpl'));
        var samlBody = _.template(samlTemplate.toString())({
            to: usernameMixedUrl,
            username: this._authOptions.username,
            password: this._authOptions.password,
            relyingParty: this._authOptions.relyingParty
        });
        return request.post(usernameMixedUrl, {
            body: samlBody,
            strictSSL: false,
            simple: false,
            headers: {
                'Content-Length': samlBody.length,
                'Content-Type': 'application/soap+xml; charset=utf-8'
            }
        })
            .then(function (data) {
            return _this.postTokenData(data);
        })
            .then(function (data) {
            var notAfter = new Date(data[0]).getTime();
            var expiresIn = parseInt(((notAfter - new Date().getTime()) / 1000).toString(), 10);
            var response = data[1];
            var authCookie = 'FedAuth=' + cookie.parse(response.headers['set-cookie'][0])[consts.FedAuth];
            AdfsCredentials.CookieCache.set(cacheKey, authCookie, expiresIn);
            return {
                headers: {
                    'Cookie': authCookie
                }
            };
        });
    };
    AdfsCredentials.prototype.postTokenData = function (xmlResponse) {
        var doc = new xmldoc.XmlDocument(xmlResponse);
        var tokenPostTemplate = fs.readFileSync(path.join(__dirname, '..', '..', '..', '..', 'templates', 'adfs_saml_token.tmpl'));
        var tokenResponseCollection = doc.childNamed('s:Body').firstChild;
        if (tokenResponseCollection.name.indexOf('Fault') !== -1) {
            throw new Error(tokenResponseCollection.toString());
        }
        var responseNamespace = tokenResponseCollection.name.split(':')[0];
        var securityTokenResponse = doc.childNamed('s:Body').firstChild.firstChild;
        var samlAssertion = securityTokenResponse.childNamed(responseNamespace + ':RequestedSecurityToken').firstChild;
        var notBefore = samlAssertion.firstChild.attr['NotBefore'];
        var notAfter = samlAssertion.firstChild.attr['NotOnOrAfter'];
        var result = _.template(tokenPostTemplate.toString())({
            created: notBefore,
            expires: notAfter,
            relyingParty: this._authOptions.relyingParty,
            token: samlAssertion.toString()
        });
        var tokenXmlDoc = new xmldoc.XmlDocument(result);
        var siteUrlParsed = url.parse(this._siteUrl);
        var rootSiteUrl = siteUrlParsed.protocol + "//" + siteUrlParsed.host;
        return Promise.all([notAfter, request.post(rootSiteUrl + "/_trust/", {
                form: {
                    'wa': 'wsignin1.0',
                    'wctx': rootSiteUrl + "/_layouts/Authenticate.aspx?Source=%2F",
                    'wresult': tokenXmlDoc.toString({ compressed: true })
                },
                resolveWithFullResponse: true,
                simple: false,
                strictSSL: false
            })]);
    };
    AdfsCredentials.CookieCache = new Cache_1.Cache();
    return AdfsCredentials;
}());
exports.AdfsCredentials = AdfsCredentials;
